@@ -62,7 +62,7 @@ int validate_frame(unsigned char *buff, int size)
         struct ready_frame *hdr; /* Ready is the shortest possible */
 
         if (size < sizeof(struct ready_frame)) {
-                diag(COMPONENT, DIAG_DEBUG, "short frame, size %d\n", size);
+                diag(COMPONENT, DIAG_DEBUG, "short frame, size %d", size);
                 return -1;
         }
 
@@ -75,12 +75,12 @@ int validate_frame(unsigned char *buff, int size)
         /* READY_* frames are shorter than others */
         if (hdr->opcode == htons(READY_QUERY) ||
             hdr->opcode == htons(READY_IND)) {
-                diag(COMPONENT, DIAG_DEBUG, "Received a %s\n", opcode2text(hdr->opcode));
+                diag(COMPONENT, DIAG_DEBUG, "Received a %s", opcode2text(hdr->opcode));
                 return 0;
         }
 
         if (size < sizeof(struct ctrl_frame)) {
-                diag(COMPONENT, DIAG_DEBUG, "short frame, size %d\n", size);
+                diag(COMPONENT, DIAG_DEBUG, "short frame, size %d", size);
                 return -1;
         }
 
@@ -97,7 +97,7 @@ static void handle_flush_req(struct ctrl_frame *f)
                 return;
         f->header.opcode = htons(LE_FLUSH_RSP);
         if (send_frame(lec_params.ctrl_direct, f, sizeof(struct ctrl_frame)) < 0)
-                diag(COMPONENT, DIAG_DEBUG, "could not send LE_FLUSH_RESPONSE\n");
+                diag(COMPONENT, DIAG_DEBUG, "could not send LE_FLUSH_RESPONSE");
 
         return;
 }
@@ -109,7 +109,7 @@ static void handle_flush_rsp(struct ctrl_frame *f)
         struct atmlec_msg msg;
 
         if (f->header.lec_id != htons(lec_params.c14_lec_id)) {
-                diag(COMPONENT, DIAG_DEBUG, "Wrong lec_id, ignoring\n");
+                diag(COMPONENT, DIAG_DEBUG, "Wrong lec_id, ignoring");
                 return;
         }
 
@@ -164,7 +164,7 @@ static int handle_le_arp_req(struct ctrl_frame *frame, int size)
         struct atmlec_msg *msg;
 
         if (frame->header.lec_id == htons(lec_params.c14_lec_id)) {
-                diag(COMPONENT, DIAG_DEBUG, "Ignoring own LE_ARP_REQUEST\n");
+                diag(COMPONENT, DIAG_DEBUG, "Ignoring own LE_ARP_REQUEST");
                 return 0;
         }
 
@@ -193,7 +193,7 @@ static int handle_le_arp_req(struct ctrl_frame *frame, int size)
                 msg->type = l_arp_update;
                 memcpy(msg->content.normal.mac_addr, frame->src_lan_dst.mac, ETH_ALEN);
                 memcpy(msg->content.normal.atm_addr, frame->src_atm_addr, ATM_ESA_LEN);
-                msg->content.normal.flag = (frame->header.flags & REMOTE_ADDRESS) ? 1 : 0;
+                msg->content.normal.flag = (frame->header.flags & htons(REMOTE_ADDRESS)) ? 1 : 0;
                 msg->content.normal.targetless_le_arp = 1;
                 msg->sizeoftlvs = sizeoftlvs;
                 if (sizeoftlvs > 0) memcpy(msg + 1, frame + 1, sizeoftlvs);
@@ -221,7 +221,7 @@ static int handle_narp_req(struct ctrl_frame *frame, int size)
 
         if (frame->header.lec_id == htons(lec_params.c14_lec_id) ||
             lec_params.c29_v2_capable == 0) {
-                diag(COMPONENT, DIAG_DEBUG, "Ignoring LE_NARP_REQUEST\n");
+                diag(COMPONENT, DIAG_DEBUG, "Ignoring LE_NARP_REQUEST");
                 return 0;
         }
 
@@ -236,7 +236,7 @@ static int handle_narp_req(struct ctrl_frame *frame, int size)
         msg->type = l_narp_req;
         memcpy(msg->content.normal.mac_addr, frame->src_lan_dst.mac, ETH_ALEN);
         memcpy(msg->content.normal.atm_addr, frame->src_atm_addr, ATM_ESA_LEN);
-        msg->content.normal.flag = (frame->header.flags & REMOTE_ADDRESS) ? 1 : 0;
+        msg->content.normal.flag = (frame->header.flags & htons(REMOTE_ADDRESS)) ? 1 : 0;
         msg->content.normal.no_source_le_narp = no_source;
         msg->sizeoftlvs = sizeoftlvs;
         if (sizeoftlvs > 0) memcpy(msg + 1, frame + 1, sizeoftlvs);
@@ -257,7 +257,7 @@ static int handle_arp_rsp(struct ctrl_frame *frame, int size)
         struct atmlec_msg *msg;
         
         if (frame->header.lec_id != htons(lec_params.c14_lec_id)) {
-                diag(COMPONENT, DIAG_DEBUG, "Wrong lec_id, ignoring\n");
+                diag(COMPONENT, DIAG_DEBUG, "Wrong lec_id, ignoring");
                 return 0;
         }
 
@@ -269,7 +269,7 @@ static int handle_arp_rsp(struct ctrl_frame *frame, int size)
         msg->type = l_arp_update;
         memcpy(msg->content.normal.mac_addr, frame->target_lan_dst.mac, ETH_ALEN);
         memcpy(msg->content.normal.atm_addr, frame->target_atm_addr, ATM_ESA_LEN);
-        msg->content.normal.flag = (frame->header.flags & REMOTE_ADDRESS) ? 1 : 0;
+        msg->content.normal.flag = (frame->header.flags & htons(REMOTE_ADDRESS)) ? 1 : 0;
         msg->sizeoftlvs = sizeoftlvs;
         if (sizeoftlvs > 0) memcpy(msg + 1, frame + 1, sizeoftlvs);
 
@@ -293,6 +293,21 @@ static int handle_topo_req(struct ctrl_frame *frame)
         return(msg_to_kernel(&msg, sizeof(struct atmlec_msg)));
 }
 
+static void handle_ready_ind(Conn_t *conn)
+{
+	struct atmlec_msg msg;
+
+	/* FIXME -- if its a receive only vcc we should not do this */
+
+        diag(COMPONENT, DIAG_DEBUG, "READY_IND, on fd %d; sending LE_FLUSH_REQ", conn->fd);
+        memset(&msg, 0, sizeof(struct atmlec_msg));
+        msg.type = l_flush_tran_id;
+        memcpy(msg.content.normal.atm_addr, conn->atm_address, ATM_ESA_LEN);
+        msg.content.normal.flag = send_flush_req(conn);
+
+        msg_to_kernel(&msg, sizeof(struct atmlec_msg));
+}
+
 /* Processes and validates incoming frames. Calls frame
  * dependant handler functions.
  * Returns < 0 for serious error
@@ -311,13 +326,14 @@ int handle_frame(Conn_t *conn, char *buff, int size)
                 handle_flush_req(frame);
                 break;
         case LE_FLUSH_RSP:
+                diag(COMPONENT, DIAG_DEBUG, "LE_FLUSH_RESPONSE, on fd %d", conn->fd);
                 handle_flush_rsp(frame);
                 break;
         case READY_QUERY:
                 handle_ready_query(conn, (struct ready_frame *)frame);
                 break;
         case READY_IND:
-                /* We can ignore these */
+                handle_ready_ind(conn);
                 break;
         case LE_ARP_REQ:
                 if (handle_le_arp_req(frame, size) < 0)
@@ -340,7 +356,7 @@ int handle_frame(Conn_t *conn, char *buff, int size)
                 break;
         default:
                 diag(COMPONENT, DIAG_ERROR,
-                     "Unknown frame, opcode 0x%x %s\n", ntohs(frame->header.opcode),
+                     "Unknown frame, opcode 0x%x %s", ntohs(frame->header.opcode),
                      opcode2text(frame->header.opcode));
                 break;
         }
@@ -362,7 +378,7 @@ void send_ready_ind(Conn_t *conn)
 
         retval = send_frame(conn, &frame, sizeof(struct ready_frame));
         if (retval < 0)
-                diag(COMPONENT, DIAG_DEBUG, "Could not send READY_IND, fd %d\n", conn->fd);
+                diag(COMPONENT, DIAG_DEBUG, "Could not send READY_IND, fd %d", conn->fd);
 
         return;
 }
@@ -420,7 +436,7 @@ void parse_tlvs(uint16_t opcode, unsigned char *tlvp, int numtlvs, int sizeoftlv
                 type = ntohl(type);
                 len  = tlvp[4];
                 tlvp += 5;
-                diag(COMPONENT, DIAG_DEBUG, "parse_tlvs: type %s len %d\n",
+                diag(COMPONENT, DIAG_DEBUG, "parse_tlvs: type %s len %d",
                      tlv2text(type), len);
                 if (tlvp + len > end_of_tlvs)
                         return; /* value too long */
@@ -459,7 +475,7 @@ static void extract_tlv_value(uint16_t opcode, uint32_t type, unsigned char *tlv
         case 2:
                 value16 = *(uint16_t *)tlvp;
                 value16 = ntohs(value16);
-                diag(COMPONENT, DIAG_DEBUG, "value of TLV %d\n", value16);
+                diag(COMPONENT, DIAG_DEBUG, "value of TLV %d", value16);
                 switch (type) {
                 case MAX_CUM_CTRL_TIMEOUT:
                         lec_params.c7_ctrl_timeout = value16;
@@ -499,7 +515,7 @@ static void extract_tlv_value(uint16_t opcode, uint32_t type, unsigned char *tlv
         case 4:
                 value32 = *(uint32_t *)tlvp;
                 value32 = ntohl(value32);
-                diag(COMPONENT, DIAG_DEBUG, "value of TLV %d\n", value32);
+                diag(COMPONENT, DIAG_DEBUG, "value of TLV %d", value32);
                 switch (type) {
                 case VCC_TIMEOUT_PERIOD:
                         lec_params.c12_vcc_timeout = value32;
@@ -537,7 +553,7 @@ static void extract_tlv_value(uint16_t opcode, uint32_t type, unsigned char *tlv
                 /* handle variable length TLVs */
                 switch(type) {
                 case CONFIG_FRAG_INFO:
-                        diag(COMPONENT, DIAG_INFO, "Got Config-Frag-Info TLV\n");
+                        diag(COMPONENT, DIAG_INFO, "Got Config-Frag-Info TLV");
                         break;
                 case LAYER3_ADDRESS:
                         /* do nothing */
@@ -552,7 +568,7 @@ static void extract_tlv_value(uint16_t opcode, uint32_t type, unsigned char *tlv
         return;
 
  whine:
-        diag(COMPONENT, DIAG_DEBUG, "Unknown TLV, type 0x%x, len %d\n", type, len);
+        diag(COMPONENT, DIAG_DEBUG, "Unknown TLV, type 0x%x, len %d", type, len);
 
         return;
 }
@@ -564,7 +580,7 @@ static void handle_x5(uint16_t opcode)
         if (!lec_params.c29_v2_capable)
                 return;
         if (opcode != ntohs(LE_CONFIG_RSP)) {
-                diag(COMPONENT, DIAG_WARN, "X5-Adjustment TLV received but not with LE_CONFIG_RSP\n");
+                diag(COMPONENT, DIAG_WARN, "X5-Adjustment TLV received but not with LE_CONFIG_RSP");
                 return;
         }
 
