@@ -115,7 +115,7 @@ int lec_configure(int lecs_method, struct sockaddr_atmsvc *manual_atm_addr,
         struct atm_qos qos;
         Conn_t *conn;
         
-        diag(COMPONENT, DIAG_DEBUG, "entering lec_configure\n");
+        diag(COMPONENT, DIAG_DEBUG, "entering lec_configure");
         
         /* initialize well known LECS addresses */
         memset(&addr_c5, 0, sizeof(struct sockaddr_atmsvc));
@@ -131,8 +131,8 @@ int lec_configure(int lecs_method, struct sockaddr_atmsvc *manual_atm_addr,
         
         init_conn_params(&sap, &qos, CONTROL_CONN);
 
-        if (lecs_method == LECS_MANUAL) {
-                diag(COMPONENT, DIAG_DEBUG, "trying manual LECS address\n");
+        if ((lecs_method == LECS_MANUAL) || (lecs_method == LECS_FROM_ILMI)) {
+                diag(COMPONENT, DIAG_DEBUG, "trying manual LECS address");
                 conn = setup_svc(manual_atm_addr, listen_addr, &sap, &qos);
                 if (conn) {
                         retval = do_lec_configure(conn);
@@ -141,7 +141,7 @@ int lec_configure(int lecs_method, struct sockaddr_atmsvc *manual_atm_addr,
                 }
                 else random_delay();
         }
-        diag(COMPONENT, DIAG_DEBUG, "trying well-known anycast LECS address\n");
+        diag(COMPONENT, DIAG_DEBUG, "trying well-known anycast LECS address");
         conn = setup_svc(&addr_c5, listen_addr, &sap, &qos);
         if (conn) {
                 retval = do_lec_configure(conn);
@@ -149,7 +149,7 @@ int lec_configure(int lecs_method, struct sockaddr_atmsvc *manual_atm_addr,
                 return retval;
         }
         else random_delay();
-        diag(COMPONENT, DIAG_DEBUG, "trying well-known LECS address\n");
+        diag(COMPONENT, DIAG_DEBUG, "trying well-known LECS address");
         conn = setup_svc(&addr_47, listen_addr, &sap, &qos);
         if (conn) {
                 retval = do_lec_configure(conn);
@@ -181,12 +181,12 @@ static int do_lec_configure(Conn_t *conn)
         }
         lec_params.c7c_current_timeout = lec_params.c7i_initial_ctrl_timeout; /* reset it */
         if (frame_size == 0) {
-                diag(COMPONENT, DIAG_INFO, "Timed out while waiting for LE_CONFIGURE_RESPONSE\n");
+                diag(COMPONENT, DIAG_INFO, "Timed out while waiting for LE_CONFIGURE_RESPONSE");
                 return -1; /* timeout */
         }
 
         if (parse_config_rsp(buff, frame_size) < 0) {
-                diag(COMPONENT, DIAG_ERROR, "Parsing LE_CONFIG_RESPONSE indicates failure\n");
+                diag(COMPONENT, DIAG_ERROR, "Parsing LE_CONFIG_RESPONSE indicates failure");
                 return -1;
         }
 
@@ -203,7 +203,7 @@ static int send_config_req(Conn_t *conn)
         struct ctrl_frame *config_req;
         char buff[MAX_CTRL_FRAME];
 
-        diag(COMPONENT, DIAG_DEBUG, "Sending LE_CONFIGURE_REQUEST\n");
+        diag(COMPONENT, DIAG_DEBUG, "Sending LE_CONFIGURE_REQUEST");
 
         /* TLVs make config frame variable length */
         frame_size = sizeof(struct ctrl_frame);
@@ -251,7 +251,7 @@ static int read_config_rsp(Conn_t *conn, char *buff, int buffsize)
 
         retval = select(conn->fd + 1, &rfds, NULL, NULL, &tv);
         if (retval < 0) {
-                diag(COMPONENT, DIAG_ERROR, "read_ctrl_rsp: select: %s\n",
+                diag(COMPONENT, DIAG_ERROR, "read_ctrl_rsp: select: %s",
                      strerror(errno));
                 return retval;
         }
@@ -259,12 +259,12 @@ static int read_config_rsp(Conn_t *conn, char *buff, int buffsize)
         
         frame_size = recv_frame(conn, buff, buffsize);
         if (frame_size < 0) {
-                diag(COMPONENT, DIAG_ERROR, "read_ctrl_rsp: recv_frame: %s\n",
+                diag(COMPONENT, DIAG_ERROR, "read_ctrl_rsp: recv_frame: %s",
                      strerror(errno));
                 return frame_size;
         }
         if (frame_size == 0) {
-                diag(COMPONENT, DIAG_ERROR, "read_ctrl_rsp: conn closed: %s\n",
+                diag(COMPONENT, DIAG_ERROR, "read_ctrl_rsp: conn closed: %s",
                      strerror(errno));
                 return -1;
         }
@@ -280,20 +280,20 @@ static int parse_config_rsp(unsigned char *buff, int size)
 {
         struct ctrl_frame *frame;
 
-        diag(COMPONENT, DIAG_DEBUG, "Parsing LE_CONFIG_RESPONSE\n");
+        diag(COMPONENT, DIAG_DEBUG, "Parsing LE_CONFIG_RESPONSE");
         if (validate_frame(buff, size) < 0) {
-                diag(COMPONENT, DIAG_ERROR, "parse_config_rsp: bad frame\n");
+                diag(COMPONENT, DIAG_ERROR, "parse_config_rsp: bad frame");
                 return -1;
         }
 
         frame = (struct ctrl_frame *)buff;
         if (frame->header.opcode != htons(LE_CONFIG_RSP)) {
-                diag(COMPONENT, DIAG_ERROR, "parse_config_rsp: not a LE_CONFIG_RESPONSE\n");
+                diag(COMPONENT, DIAG_ERROR, "parse_config_rsp: not a LE_CONFIG_RESPONSE");
                 return -1;
         }
         
         if (frame->header.status != 0) {
-                diag(COMPONENT, DIAG_ERROR, "LECS said: %s\n",
+                diag(COMPONENT, DIAG_ERROR, "LECS said: %s",
                      status2text(frame->header.status));
                 return -1;
         }
@@ -332,10 +332,10 @@ int les_connect(int lecs_method, struct sockaddr_atmsvc *manual_atm_addr,
         char buff[MAX_CTRL_FRAME];
         int frame_size = 0; /* shut up, GCC */
 
-        diag(COMPONENT, DIAG_DEBUG, "Entering Join phase\n");
+        diag(COMPONENT, DIAG_DEBUG, "Entering Join phase");
 
         if (lecs_method == LECS_NONE) {
-                diag(COMPONENT, DIAG_DEBUG, "Skipping LECS, connecting straight to LES\n");
+                diag(COMPONENT, DIAG_DEBUG, "Skipping LECS, connecting straight to LES");
                 memcpy(les_addr.sas_addr.prv, manual_atm_addr->sas_addr.prv, ATM_ESA_LEN);
         } else memcpy(les_addr.sas_addr.prv, lec_params.c9_les_atm_addr, ATM_ESA_LEN);
 	*les_addr.sas_addr.pub = 0;
@@ -343,13 +343,13 @@ int les_connect(int lecs_method, struct sockaddr_atmsvc *manual_atm_addr,
         init_conn_params(&sap, &qos, CONTROL_CONN);
         lec_params.ctrl_direct = setup_svc(&les_addr, listen_addr, &sap, &qos);
         if (lec_params.ctrl_direct == NULL) {
-                diag(COMPONENT, DIAG_ERROR, "Control direct SVC failed\n");
+                diag(COMPONENT, DIAG_ERROR, "Control direct SVC failed");
                 random_delay();
                 return -1;
         }
         lec_params.ctrl_listen = create_listensocket(listen_addr, &sap, &qos);
         if (lec_params.ctrl_listen == NULL) {
-                diag(COMPONENT, DIAG_ERROR, "Control distribute listen socket failed\n");
+                diag(COMPONENT, DIAG_ERROR, "Control distribute listen socket failed");
                 random_delay();
                 return -1;
         }
@@ -357,13 +357,13 @@ int les_connect(int lecs_method, struct sockaddr_atmsvc *manual_atm_addr,
         lec_params.c7c_current_timeout = lec_params.c7i_initial_ctrl_timeout; /* reset it */
         while (lec_params.c7c_current_timeout <= lec_params.c7_ctrl_timeout) {
                 if (send_join_req(lec_params.ctrl_direct) < 0) {
-                        diag(COMPONENT, DIAG_ERROR, "Sending LE_JOIN_REQUEST failed\n");
+                        diag(COMPONENT, DIAG_ERROR, "Sending LE_JOIN_REQUEST failed");
                         random_delay();
                         return -1;
                 }
                 frame_size = read_join_rsp(buff, sizeof(buff));
                 if (frame_size < 0) {
-                        diag(COMPONENT, DIAG_ERROR, "Receiving LE_JOIN_RESPONSE failed\n");
+                        diag(COMPONENT, DIAG_ERROR, "Receiving LE_JOIN_RESPONSE failed");
                         random_delay();
                         return -1;
                 } else if (frame_size == 0) /* timeout */
@@ -373,12 +373,12 @@ int les_connect(int lecs_method, struct sockaddr_atmsvc *manual_atm_addr,
         }
         lec_params.c7c_current_timeout = lec_params.c7i_initial_ctrl_timeout; /* reset it */
         if (frame_size == 0) {
-                diag(COMPONENT, DIAG_ERROR, "LE_JOIN_RESPONSE timed out\n");
+                diag(COMPONENT, DIAG_ERROR, "LE_JOIN_RESPONSE timed out");
                 return -1;
         }
         
         if (parse_join_rsp(buff, frame_size) < 0) {
-                diag(COMPONENT, DIAG_ERROR, "Parsing LE_JOIN_RESPONSE failed\n");
+                diag(COMPONENT, DIAG_ERROR, "Parsing LE_JOIN_RESPONSE failed");
                 return -1;
         }
 
@@ -472,7 +472,7 @@ static int read_join_rsp(char *buff, int buffsize)
         
         retval = select(n, &rfds, NULL, NULL, &tv);
         if (retval < 0) {
-                diag(COMPONENT, DIAG_ERROR, "get_join_rsp: select: %s\n",
+                diag(COMPONENT, DIAG_ERROR, "get_join_rsp: select: %s",
                      strerror(errno));
                 return -1;
         }
@@ -485,15 +485,15 @@ static int read_join_rsp(char *buff, int buffsize)
                 /* Control Direct was changed */
                 frame_size = recv_frame(lec_params.ctrl_direct, buff, buffsize);
                 if (frame_size < 0) {
-                        diag(COMPONENT, DIAG_ERROR, "get_join_rsp: recv_frame: %s\n",
+                        diag(COMPONENT, DIAG_ERROR, "get_join_rsp: recv_frame: %s",
                              strerror(errno));
                         return -1;
                 }
                 if (frame_size == 0) {
-                        diag(COMPONENT, DIAG_ERROR, "get_join_rsp: Control direct VCC closed\n");
+                        diag(COMPONENT, DIAG_ERROR, "get_join_rsp: Control direct VCC closed");
                         return -1;
                 }
-                diag(COMPONENT, DIAG_DEBUG, "LE_JOIN_RESPONSE over Control direct VCC\n");
+                diag(COMPONENT, DIAG_DEBUG, "LE_JOIN_RESPONSE over Control direct VCC");
         }
         if (FD_ISSET(dist->fd, &rfds)) {
                 /* Event in listen socket or Control Distribute */
@@ -501,10 +501,10 @@ static int read_join_rsp(char *buff, int buffsize)
                         /* Connection to control listen */
                         lec_params.ctrl_dist = accept_conn(lec_params.ctrl_listen);
                         if (lec_params.ctrl_dist == NULL) {
-                                diag(COMPONENT, DIAG_ERROR, "accept of Ctrl distribute failed\n");
+                                diag(COMPONENT, DIAG_ERROR, "accept of Ctrl distribute failed");
                                 return -1;
                         }
-                        diag(COMPONENT, DIAG_DEBUG, "Closing listen socket for Ctrl distribute VCC\n");
+                        diag(COMPONENT, DIAG_DEBUG, "Closing listen socket for Ctrl distribute VCC");
                         close_connection(lec_params.ctrl_listen);
                         lec_params.ctrl_listen = NULL;
 
@@ -514,15 +514,15 @@ static int read_join_rsp(char *buff, int buffsize)
                 /* Event in Control distribute */
                 frame_size = recv_frame(lec_params.ctrl_dist, buff, buffsize);
                 if (frame_size < 0) {
-                        diag(COMPONENT, DIAG_ERROR, "get_join_rsp: recv_frame: %s\n",
+                        diag(COMPONENT, DIAG_ERROR, "get_join_rsp: recv_frame: %s",
                              strerror(errno));
                         return -1;
                 }
                 if (frame_size == 0) {
-                        diag(COMPONENT, DIAG_ERROR, "Control distribute VCC closed\n");
+                        diag(COMPONENT, DIAG_ERROR, "Control distribute VCC closed");
                         return -1;
                 }
-                diag(COMPONENT, DIAG_DEBUG, "LE_JOIN_RESPONSE over Control distribute VCC\n");
+                diag(COMPONENT, DIAG_DEBUG, "LE_JOIN_RESPONSE over Control distribute VCC");
         }
 
         return frame_size;
@@ -536,7 +536,7 @@ static int parse_join_rsp(unsigned char *buff, int size)
 {
         struct ctrl_frame *frame;
 
-        diag(COMPONENT, DIAG_DEBUG, "Parsing LE_JOIN_RESPONSE\n");
+        diag(COMPONENT, DIAG_DEBUG, "Parsing LE_JOIN_RESPONSE");
         if (validate_frame(buff, size) < 0)
                 return -1;
 
@@ -545,7 +545,7 @@ static int parse_join_rsp(unsigned char *buff, int size)
                 return -1;
         
         if (frame->header.status != 0) {
-                diag(COMPONENT, DIAG_ERROR, "LES said: %s\n",
+                diag(COMPONENT, DIAG_ERROR, "LES said: %s",
                      status2text(frame->header.status));
                 return -1;
         }
@@ -560,13 +560,13 @@ static int parse_join_rsp(unsigned char *buff, int size)
         lec_params.c14_lec_id = ntohs(frame->header.lec_id);
 
         if (!(frame->header.flags & htons(V2_REQUIRED)) && lec_params.c29_v2_capable) {
-                diag(COMPONENT, DIAG_INFO, "LES did not return V2 Required Flag, acting as V1 client\n");
+                diag(COMPONENT, DIAG_INFO, "LES did not return V2 Required Flag, acting as V1 client");
                 lec_params.c29_v2_capable = 0;
         }
         if (!(frame->header.flags & htons(V2_REQUIRED)) &&
             (lec_params.c3_max_frame_size == MTU_1580)) {
                 /* Against spec, but we'll accept the MTU and clear the flag */
-                diag(COMPONENT, DIAG_ERROR, "LES not LANEv2 but uses MTU of 1580 bytes\n");
+                diag(COMPONENT, DIAG_ERROR, "LES not LANEv2 but uses MTU of 1580 bytes");
                 lec_params.c29_v2_capable = 0;
         }
         parse_tlvs(frame->header.opcode, buff + sizeof(struct ctrl_frame),
@@ -609,19 +609,19 @@ int bus_connect(void)
                 memcpy(frame->target_lan_dst.mac, bus_mac, ETH_ALEN);
                 retval = send_frame(lec_params.ctrl_direct, frame, sizeof(struct ctrl_frame));
                 if (retval < 0) {
-                        diag(COMPONENT, DIAG_ERROR, "LE_ARP_REQUEST for BUS failed\n");
+                        diag(COMPONENT, DIAG_ERROR, "LE_ARP_REQUEST for BUS failed");
                         return -1;
                 }
                 retval = get_bus_addr(&bus_addr);
                 if (retval < 0) {
-                        diag(COMPONENT, DIAG_ERROR, "LE_ARP_RESPONSE for BUS failed\n");
+                        diag(COMPONENT, DIAG_ERROR, "LE_ARP_RESPONSE for BUS failed");
                         return -1;
                 } else if (retval > 0)
                         break; /* got it */
                 tries--;
         }
         if (tries == 0) {
-                diag(COMPONENT, DIAG_ERROR, "LE_ARP_RESPONSE for BUS timed out\n");
+                diag(COMPONENT, DIAG_ERROR, "LE_ARP_RESPONSE for BUS timed out");
                 return -1;
         }
 
@@ -633,22 +633,22 @@ int bus_connect(void)
         init_conn_params(&sap, &qos, MCAST_CONN);
         lec_params.mcast_listen = create_listensocket(&listen_addr, &sap, &qos);
         if (lec_params.mcast_listen == NULL) {
-                diag(COMPONENT, DIAG_ERROR, "Listen socket for BUS failed\n");
+                diag(COMPONENT, DIAG_ERROR, "Listen socket for BUS failed");
                 return -1;
         }
 
         lec_params.mcast_send = setup_svc(&bus_addr, &listen_addr, &sap, &qos);
         if (lec_params.mcast_send == NULL) {
-                diag(COMPONENT, DIAG_ERROR, "Connect to BUS failed\n");
+                diag(COMPONENT, DIAG_ERROR, "Connect to BUS failed");
                 return -1;
         }
         /* Default Multicast send VCC to BUS ready, notify kernel */
         if (ioctl(lec_params.mcast_send->fd, ATMLEC_MCAST, lec_params.itf_num) < 0) {
-                diag(COMPONENT, DIAG_FATAL, "Can't change socket into LE mcast socket: %s\n", strerror(errno));
+                diag(COMPONENT, DIAG_FATAL, "Can't change socket into LE mcast socket: %s", strerror(errno));
                 return -1;
         }
 
-        diag(COMPONENT, DIAG_DEBUG, "About to wait for BUS to connect\n");
+        diag(COMPONENT, DIAG_DEBUG, "About to wait for BUS to connect");
         tv.tv_sec = lec_params.c7_ctrl_timeout;
         tv.tv_usec = 0;
         FD_ZERO(&rfds);
@@ -656,26 +656,26 @@ int bus_connect(void)
         n = lec_params.mcast_listen->fd + 1;
         retval = select(n, &rfds, NULL, NULL, &tv);
         if (retval == 0) {
-                diag(COMPONENT, DIAG_ERROR, "BUS connect to Multicast Forward listen socket timed out\n");
+                diag(COMPONENT, DIAG_ERROR, "BUS connect to Multicast Forward listen socket timed out");
                 return -1;
         }
         if (retval < 0) {
-                diag(COMPONENT, DIAG_ERROR, "while waiting for Multicast Forward VCC: select: %s\n",
+                diag(COMPONENT, DIAG_ERROR, "while waiting for Multicast Forward VCC: select: %s",
                      strerror(errno));
                 return -1;
         }
         mcast_fwd = accept_conn(lec_params.mcast_listen);
         if (mcast_fwd == NULL) {
-                diag(COMPONENT, DIAG_ERROR, "BUS connect to Multicast Forward listen socket failed\n");
+                diag(COMPONENT, DIAG_ERROR, "BUS connect to Multicast Forward listen socket failed");
                 return -1;
         }
         
         memcpy(ioc_data.atm_addr, mcast_fwd->atm_address, ATM_ESA_LEN);
         ioc_data.dev_num = lec_params.itf_num;
         ioc_data.receive = 2; /* Multicast distribute */
-        diag(COMPONENT, DIAG_DEBUG, "About to notify kernel about Multicast Forward VCC\n");
+        diag(COMPONENT, DIAG_DEBUG, "About to notify kernel about Multicast Forward VCC");
         if (ioctl(mcast_fwd->fd, ATMLEC_DATA, &ioc_data) < 0) {
-                diag(COMPONENT, DIAG_DEBUG, "Could not notify kernel: %s\n", strerror(errno));
+                diag(COMPONENT, DIAG_DEBUG, "Could not notify kernel: %s", strerror(errno));
                 return -1;
         }
 
@@ -715,25 +715,25 @@ static int get_bus_addr(struct sockaddr_atmsvc *addr)
                         continue; /* back to waiting */
                 }
                 if (retval < 0) {
-                        diag(COMPONENT, DIAG_ERROR, "get_bus_addr: select: %s\n", strerror(errno));
+                        diag(COMPONENT, DIAG_ERROR, "get_bus_addr: select: %s", strerror(errno));
                         return -1;
                 }
                 if (FD_ISSET(lec_params.ctrl_direct->fd, &rfds)) {
-                        diag(COMPONENT, DIAG_DEBUG, "get_bus_addr: ctrl.direct changed:\n");
+                        diag(COMPONENT, DIAG_DEBUG, "get_bus_addr: ctrl.direct changed:");
                         retval = read_bus_arp(lec_params.ctrl_direct, addr, buff, sizeof(buff));
                         if (retval < 0) return -1;
                         if (retval > 0) return retval;
                 }
                 if (lec_params.ctrl_dist != NULL && FD_ISSET(lec_params.ctrl_dist->fd, &rfds)) {
-                        diag(COMPONENT, DIAG_DEBUG, "get_bus_addr: ctrl.dist changed:\n");
+                        diag(COMPONENT, DIAG_DEBUG, "get_bus_addr: ctrl.dist changed:");
                         retval = read_bus_arp(lec_params.ctrl_dist, addr, buff, sizeof(buff));
                         if (retval < 0) return -1;
                         if (retval > 0) return retval;
                 }
-                diag(COMPONENT, DIAG_DEBUG, "get_bus_addr: consumed a packet\n");
+                diag(COMPONENT, DIAG_DEBUG, "get_bus_addr: consumed a packet");
         }
 
-        diag(COMPONENT, DIAG_ERROR, "Timeout while waiting for BUS LE_ARP response\n");
+        diag(COMPONENT, DIAG_ERROR, "Timeout while waiting for BUS LE_ARP response");
         return 0;
 }
 /*
@@ -747,11 +747,11 @@ static int read_bus_arp(Conn_t *conn, struct sockaddr_atmsvc *addr, char *buff,
 
         frame_size = recv_frame(conn, buff, buffsize);
         if (frame_size == 0) {
-                diag(COMPONENT, DIAG_ERROR, "LES Control connection closed\n");
+                diag(COMPONENT, DIAG_ERROR, "LES Control connection closed");
                 return -1;
         }
         if (frame_size < 0) {
-                diag(COMPONENT, DIAG_ERROR, "get_bus_arp: recv_frame: %s\n", strerror(errno));
+                diag(COMPONENT, DIAG_ERROR, "get_bus_arp: recv_frame: %s", strerror(errno));
                 return -1;
         }
         frame = (struct ctrl_frame *)buff;
