@@ -11,6 +11,8 @@
 #include <stdio.h>
 #include <string.h>
 #include <syslog.h>
+#include <signal.h>
+#include <time.h>
 
 #include "atmd.h"
 
@@ -126,7 +128,11 @@ void vdiag(const char *component,int severity,const char *fmt,va_list ap)
     COMPONENT *walk;
     FILE *to;
     char buffer[MAX_DIAG_MSG+1];
+    struct timeval tv;
     int i;
+#define DATESTR_LEN 32
+    char datestr[DATESTR_LEN];
+    struct tm tm;
 
     for (walk = components; walk; walk = walk->next)
 	if (!strcmp(walk->name,component)) break;
@@ -139,8 +145,14 @@ void vdiag(const char *component,int severity,const char *fmt,va_list ap)
 	syslog(sev2prio[i+1],"%s: %s",component,buffer);
     }
     else {
-	if (app_name) fprintf(to,"%s:%s: ",app_name,component);
-	else fprintf(to,"%s: ",component);
+	gettimeofday(&tv,NULL);
+	localtime_r((time_t *) &tv.tv_sec, &tm);
+	strftime(datestr, DATESTR_LEN, "%b %d %T", &tm);
+	if (app_name)
+	    fprintf(to, "%s.%06lu %s:%s: ",
+		    datestr, (unsigned long) tv.tv_usec, app_name, component);
+	else
+	    fprintf(to,"%s.%06lu %s: ", datestr, (unsigned long) tv.tv_usec, component);
 	vfprintf(to,fmt,ap);
 	fputc('\n',to);
 	fflush(to);
