@@ -2,10 +2,6 @@
  
 /* Written 1995 by Werner Almesberger, EPFL-LRC */
 
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <stdlib.h>
 #include <stdarg.h>
 #include <stdio.h>
