@@ -30,6 +30,111 @@
 #define COMPONENT "KERNEL"
 
 
+#ifdef MULTIPOINT
+int send_dropparty(SOCKET *sock)
+{
+	Q_DSC dsc;
+	int size;
+
+	q_create(&dsc, q_buffer, MAX_Q_MSG);
+	q_assign(&dsc, QF_msg_type, ATM_MSG_DROP_PARTY);
+	q_assign(&dsc, QF_call_ref, sock->call_ref);
+	q_assign(&dsc, QF_ep_ref, sock->ep_ref);
+	q_assign(&dsc, QF_cause, ATM_CV_NORMAL_UNSPEC);
+	if ((size = q_close(&dsc)) < 0)
+		return -EINVAL;
+
+	to_signaling(sock->sig, q_buffer, size);
+	START_TIMER(sock, T398);
+	return 0;
+}
+
+
+int send_addparty(SOCKET *sock, struct sockaddr_atmsvc *svc)
+{
+    SOCKET *walk = NULL;
+    Q_DSC dsc;
+    struct sockaddr_atmsvc *local = NULL;
+    int error = 0,size,max_ep_ref = 0;
+    char buf[80];
+
+    diag(COMPONENT, DIAG_INFO, "send_addparty");
+    q_create(&dsc,q_buffer,MAX_Q_MSG);
+
+    /* think this next line is already done */
+    /* sock->sig = route_remote(&sock->remote); */
+
+    /* set the calling party address */
+    if (sock->sig && sock->sig->mode == sm_switch) local = &sock->local;
+    if (atmsvc_addr_in_use(sock->local) && !local) {
+	sock->sig = route_local(&sock->local);
+	if (sock->sig) local = &sock->local;
+	else {
+	    error = -EADDRNOTAVAIL;
+	    diag(COMPONENT,DIAG_WARN,"local address no longer available");
+	}
+    }
+    else {
+	if (!sock->sig) error = -EHOSTUNREACH;
+	else {
+	    if (!local) local = get_local(sock->sig);
+	    if (!local) local = get_local(NULL);
+	    if (local) sock->local = *local;
+	    else {
+		error = -EADDRNOTAVAIL;
+		diag(COMPONENT,DIAG_ERROR,"no local address");
+	    }
+	}
+    }
+    if (local) {
+	if (*local->sas_addr.pub) {
+	    q_assign(&dsc,QF_cgpn_plan,ATM_NP_E164);
+	    q_assign(&dsc,QF_cgpn_type,ATM_TON_INTRNTNL);
+	    q_write(&dsc,QF_cgpn,(void *) local->sas_addr.pub,
+	      strlen(local->sas_addr.pub));
+	}
+	else {
+	    q_assign(&dsc,QF_cgpn_plan,ATM_NP_AEA);
+	    q_assign(&dsc,QF_cgpn_type,ATM_TON_UNKNOWN);
+	    q_write(&dsc,QF_cgpn,(void *) local->sas_addr.prv,ATM_ESA_LEN);
+	}
+    }
+
+    /* what to do if error == -EADDRNOTAVAIL ? */
+
+    /* find the largest endpoint reference in the list */
+    for (walk = sockets; walk->next; walk = walk->next) {
+	if ((walk->call_ref == sock->call_ref) &&
+		(walk->ep_ref > max_ep_ref)) max_ep_ref = walk->ep_ref;
+    }
+    sock->ep_ref = max_ep_ref + 1;
+
+    q_assign(&dsc,QF_call_ref,sock->call_ref);
+    q_assign(&dsc,QF_aal_type,5); /* AAL 5 */
+    q_assign(&dsc,QF_msg_type,ATM_MSG_ADD_PARTY);
+
+    diag(COMPONENT, DIAG_INFO, "send_addparty: sap_encode");
+    if (atm2text(buf,MAX_ATM_ADDR_LEN+1,(struct sockaddr *)svc, pretty) >= 0)
+	    diag(COMPONENT, DIAG_INFO, "send_addparty -> %s", buf);
+    else
+	    diag(COMPONENT, DIAG_INFO, "send_addparty: invalid nsap address");
+    error = sap_encode(&dsc,svc,&sock->sap,NULL,sock->sig->uni,sock->sig->max_rate);
+    diag(COMPONENT, DIAG_INFO, "send_addparty: sap_encode error = %d", error);
+
+    q_assign(&dsc, QF_ep_ref, sock->ep_ref);
+    error = 0;
+    if ((size = q_close(&dsc)) < 0) {
+	error = -EINVAL;
+    } else if (!error) {
+		to_signaling(sock->sig,q_buffer,size);
+		new_state(sock,ss_connecting);
+		return sock->ep_ref;
+    }
+    return error;
+}
+#endif
+
+
 static int send_setup(SOCKET *sock)
 {
     static unsigned long call_ref = 0;
@@ -96,7 +201,19 @@ static int send_setup(SOCKET *sock)
 	error = sap_encode(&dsc,&sock->remote,&sock->sap,&sock->qos,
 	  sock->sig->uni,sock->sig->max_rate);
     q_assign(&dsc,QF_bearer_class,16); /* BCOB-X */
+#ifndef MULTIPOINT
     q_assign(&dsc,QF_upcc,0); /* p2p */
+#else
+    if (sock->ct == p2p) {
+	diag(COMPONENT, DIAG_INFO, "setting configuration to point-to-point");
+	q_assign(&dsc,QF_upcc,0); /* p2p */
+    } else if (sock->ct == p2mp) {
+	diag(COMPONENT, DIAG_INFO, "setting configuration to multipoint");
+	q_assign(&dsc, QF_upcc, 1);   /* p2mp */
+	q_assign(&dsc, QF_ep_ref, 0); /* initial setup always leaf 0 */
+	sock->ep_ref = 0;
+    }
+#endif
 #if !defined(UNI30) || defined(DYNAMIC_UNI)
     if (!error && sock->sig->uni != S_UNI30)
 	q_assign(&dsc,QF_qos_cs,Q2931_CS_ITU);
@@ -230,6 +347,11 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 	    sock->sap = msg->sap;
 	    sock->state = ss_connecting;
 	    sock->pvc = msg->pvc;
+#ifdef MULTIPOINT
+	    sock->ct = p2p;
+	    if (msg->session > 0)
+		sock->ct = p2mp;
+#endif
 	    error = send_setup(sock);
 	    if (error) {
 		SEND_ERROR(msg->vcc,error);
@@ -313,7 +435,7 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		case ss_zombie:
 		    send_close(sock);
 		    /* fall through */
-		case ss_wait_close:
+		case ss_wait_close:	/* network requested close */
 		    free_sock(sock);
 		    /* fall through */
 		case ss_null:
@@ -358,6 +480,7 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		    return;
 		case ss_rel_ind:
 		    send_release_complete(sock->sig,sock->call_ref,0); /* @@@ */
+		    diag(COMPONENT, DIAG_INFO, "ss_rel_ind");
 		    free_sock(sock);
 		    return;
 		default:
@@ -477,6 +600,86 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		default:
 		    return; /* ignore stray as_error */
 	    }
+#endif
+#ifdef MULTIPOINT
+	case as_dropparty: {
+		int leaves;
+
+		/* This should send a RELEASE if there is only
+		 * one party left on the call
+		 */
+
+		if (sock == NULL) {
+			/* leaf was probably not found */
+			send_kernel(msg->vcc, kptr_null, as_dropparty, -1,
+				    NULL, NULL, NULL, NULL, NULL);
+			return;
+		}
+		leaves = count_leaves(&sock->id);
+		if ((sock->state == ss_connected) ||
+		    (sock->state == ss_connecting)) {
+			/* rely on short circuit boolean eval */
+			if ((leaves > 1) && (send_dropparty(sock) < 0)) {
+				send_kernel(sock->id, kptr_null, as_error, 0,
+					    NULL, NULL, NULL, NULL, NULL);
+			} else if (leaves > 1) {
+				new_state(sock, ss_rel_req);
+			} else if (leaves == 1) {
+				send_release(sock, sock->sig->uni
+					!= S_UNI30 ? ATM_CV_NORMAL_CLEAR :
+					ATM_CV_NORMAL_UNSPEC);
+			}
+		} else {
+			send_kernel(sock->id, kptr_null, as_error, 0,
+				    NULL, NULL, NULL, NULL, NULL);
+		}
+		return;
+	}
+
+	case as_addparty:
+		{
+		int leafid = -1;
+		SOCKET *ns = NULL;
+
+		if (sock == NULL) {
+			diag(COMPONENT, DIAG_INFO,
+				"*sock is null.  Not adding party.");
+		} else {
+			/* Create a new socket for keeping state information
+			 * that has the same "id" and call reference value
+			 * as *sock, but should have a different endpoint
+			 * reference.  Need to make sure that all routines
+			 * that search for SOCKETS by id also compare endpoint
+			 * references */
+
+			ns = new_sock(msg->vcc);
+			ns->call_ref = sock->call_ref;
+
+			ns->local = msg->local;
+			ns->remote = msg->svc;
+			ns->qos = msg->qos;
+			ns->sap = msg->sap;
+			ns->state = ss_connecting;
+			ns->pvc = msg->pvc;
+			ns->sig = route_remote(&ns->remote);
+			leafid = send_addparty(ns, &msg->svc);
+			START_TIMER(ns,T399);
+		}
+
+		if (leafid > 0) {
+			/* don't do anything here because we will notify
+			 * the kernel of success or failure after the
+			 * network responds with ATM_MSG_ADD_PARTY_ACK or
+			 * ATM_MSG_ADD_PARTY_REJ */
+		} else {
+			/* notify kernel of failure because the addparty
+			 * request was never sent */
+			diag(COMPONENT, DIAG_INFO, "send_addparty failed");
+			send_kernel(ns->id, kptr_null, as_error, 0,
+				    NULL, NULL, NULL, NULL, NULL);
+		}
+		return;
+		}
 #endif
 	default:
     	    diag(COMPONENT,DIAG_WARN,"invalid message %d",(int) msg->type);
@@ -577,6 +780,17 @@ void from_kernel(struct atmsvc_msg *msg,int size)
 	      (curr->state == ss_listening || curr->state == ss_listen_zombie))
 		break;
     }
+#ifdef MULTIPOINT
+    else if (msg->type == as_dropparty) {
+	/* check the endpoint reference while we search */
+	dispatcher = dispatch;
+	for (curr = sockets; curr; curr = curr->next) {
+		if (kptr_eq(&msg->vcc, &curr->id) &&
+		    (curr->ep_ref == msg->reply)) break;
+	}
+	if (curr == NULL) diag(COMPONENT, DIAG_INFO, "leaf not found");
+    }
+#endif
     else {
 	dispatcher = dispatch;
 	for (curr = sockets; curr; curr = curr->next)
