@@ -516,7 +516,13 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 			/* fall through */
 		    case ss_wait_rel:
 			STOP_TIMER(sock);
-			free_sock(sock);
+#ifdef MULTIPOINT
+			/* Need to free all endpoints if this is a pmp call */
+			if (sock->ct == p2mp)
+				free_leaves(&sock->id);
+			else
+#endif
+				free_sock(sock);
 			return;
 #if defined(Q2963_1) || defined(DYNAMIC_UNI)
 		    case ss_mod_req:
@@ -573,6 +579,46 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	case ATM_MSG_NOTIFY:
 		/* silently ignore this junk */
 	    return;
+#ifdef MULTIPOINT
+	case ATM_MSG_ADD_PARTY_REJ:
+	    STOP_TIMER(sock);
+	    send_drop_party_ack(sock->sig,sock->call_ref,sock->ep_ref,ATM_CV_CALL_REJ);
+	    send_kernel(sock->id, kptr_null, as_addparty, -1,
+			NULL, NULL, NULL, NULL, NULL);
+	    diag(COMPONENT, DIAG_INFO, "addparty failed");
+		 /* If no other leaves, should close entire call */
+	    free_sock(sock);
+	    return;
+	case ATM_MSG_ADD_PARTY_ACK:
+	    STOP_TIMER(sock);
+	    send_kernel(sock->id, kptr_null, as_addparty, sock->ep_ref,
+			NULL, NULL, NULL, NULL, NULL);
+	    diag(COMPONENT, DIAG_INFO, "addparty succeeded");
+	    new_state(sock, ss_connected);
+	    return;
+	case ATM_MSG_DROP_PARTY_ACK:
+	    STOP_TIMER(sock);
+	    send_kernel(sock->id, kptr_null, as_dropparty, 0,
+			NULL, NULL, NULL, NULL, NULL);
+	    diag(COMPONENT, DIAG_INFO, "dropparty succeeded");
+	    free_sock(sock);
+	    return;
+        case ATM_MSG_DROP_PARTY:
+	    diag(COMPONENT, DIAG_INFO, "received dropparty"); 
+	    switch (sock->state) {
+		case ss_connecting:
+		case ss_connected:
+		case ss_rel_req:
+		case ss_wait_rel:
+		    send_drop_party_ack(sock->sig, sock->call_ref, sock->ep_ref,
+					ATM_CV_NORMAL_CLEAR);
+		    break;
+		default:
+		    send_drop_party_ack(sock->sig, sock->call_ref, sock->ep_ref, 0);
+	    }
+	    free_sock(sock);
+	    return;
+#endif
 #if defined(Q2963_1) || defined(DYNAMIC_UNI)
 /*
  * Buglet ahead: should actually test "call_state"
@@ -705,6 +751,7 @@ static void process_uni(SIG_ENTITY *sig,void *msg)
 {
     SOCKET *curr;
     unsigned long call_ref;
+    unsigned short ep_ref;
     unsigned char mid;
 
     call_ref = q_fetch(&in_dsc,QF_call_ref)^0x800000;
@@ -751,8 +798,18 @@ static void process_uni(SIG_ENTITY *sig,void *msg)
 	diag(COMPONENT,DIAG_ERROR,"unrecognized global call ref");
 	return;
     }
-    for (curr = sockets; curr; curr = curr->next)
-	if (curr->sig == sig && curr->call_ref == call_ref) break;
+
+#ifdef MULTIPOINT
+    if (q_present(&in_dsc,QF_ep_ref)) {
+	    ep_ref = cvt_ep_ref(sig,q_fetch(&in_dsc,QF_ep_ref));
+	    for (curr = sockets; curr; curr = curr->next)
+		if (curr->sig == sig &&
+		    curr->call_ref == call_ref &&
+		    curr->ep_ref == ep_ref) break;
+    } else
+#endif
+	    for (curr = sockets; curr; curr = curr->next)
+		if (curr->sig == sig && curr->call_ref == call_ref) break;
     diag(COMPONENT,DIAG_DEBUG,"FROM SAAL %d.%d.%d: %s (0x%02X) CR 0x%06lx for "
       "%s",S_PVC(sig),mid2name(((unsigned char *) msg)[5]),
       ((unsigned char *) msg)[5],call_ref,curr ? kptr_print(&curr->id) : "?");
@@ -766,8 +823,6 @@ static void process_uni(SIG_ENTITY *sig,void *msg)
     }
     if (curr && q_present(&in_dsc,QF_ep_ref) && mid != ATM_MSG_ADD_PARTY &&
       mid != ATM_MSG_DROP_PARTY_ACK) {
-	unsigned short ep_ref;
-
 	ep_ref = cvt_ep_ref(sig,q_fetch(&in_dsc,QF_ep_ref));
 	if (curr->ep_ref != ep_ref) {
 	    send_drop_party_ack(sig,call_ref,ep_ref,ATM_CV_INV_EPR);
