@@ -45,7 +45,7 @@
 #endif
 
 #ifndef lint
-static char RCSid[] = "ttcp.c $Revision: 1.1.2.1 $";
+static char RCSid[] = "ttcp.c $Revision: 1.2.2.1 $";
 #endif
 
 /*#define CPU_USAGE*/	/* print out CPU usage numbers? */
@@ -55,9 +55,16 @@ static char RCSid[] = "ttcp.c $Revision: 1.1.2.1 $";
 /* #define SYSV */	/* required on SGI IRIX releases before 3.3 */
 
 #include <stdio.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <string.h>
+#include <getopt.h>
 #include <signal.h>
 #include <ctype.h>
 #include <errno.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -120,9 +127,6 @@ static struct timeval stop_time;	/* Time at which timing stopped */
 static struct rusage ru0;		/* Resource utilization at the start */
 
 struct hostent *addr;
-extern int errno;
-extern int optind;
-extern char *optarg;
 
 char Usage[] = "\
 Usage: ttcp -t [-options] host [ < in ]\n\
@@ -158,24 +162,25 @@ double nbytes;			/* bytes on net */
 unsigned long numCalls;		/* # of I/O system calls */
 double cput, realt;		/* user, real time (seconds) */
 
-void err();
-void mes();
-int pattern();
-void prep_timer();
-double read_timer();
-int Nread();
-int Nwrite();
-void delay();
-int mread();
-char *outfmt();
-static void prusage();
-static void tvadd();
-static void tvsub();
-static void psecs();
+void err(const char *);
+void mes(const char *);
+void pattern(char *, int);
+void prep_timer(void);
+double read_timer(char *,  int);
+int Nread(int, char *, int);
+int Nwrite(int, char *, int);
+void delay(long);
+int mread(int, char *, unsigned);
+char *outfmt(double);
+static void prusage(struct rusage *, struct rusage *, struct timeval *, struct timeval *, char *);
+static void tvadd(struct timeval *, struct timeval *, struct timeval *);
+static void tvsub(struct timeval *, struct timeval *, struct timeval *);
+static void psecs(long, char*);
 
 void
-sigpipe()
+sigpipe(int sig)
 {
+    return;
 }
 
 
@@ -183,9 +188,8 @@ sigpipe()
  * main - 
  *-------------------------------------------------------------------------
  */
-main(argc,argv)
-int argc;
-char **argv;
+int
+main(int argc, char **argv)
 {
     struct timeval td;
     unsigned long addr_tmp;
@@ -275,20 +279,22 @@ int no_check = 0;
 	}
     }
 
-    if (port_name)
-	if (atm) goto usage;
-	else {
+    if (port_name) {
+	if (atm) {
+            goto usage;
+	} else {
 	    struct servent *se;
 
 	    se = getservbyname(port_name,udp ? "udp" : "tcp");
 	    if (se) port = ntohs(se->s_port);
 	    else {
-		const char *end;
+		char *end;
 
 		port = strtoul(port_name,&end,0);
 		if (*end) goto usage;
 	    }
 	}     
+    }
 
     host = argv[optind];
 
@@ -406,36 +412,9 @@ int no_check = 0;
 	    err("setsockopt SO_ATMSAP");
     }
 #endif
-    /* set socket buffer size */
-#if defined(SO_SNDBUF) || defined(SO_RCVBUF)
-    if (sockbufsize) {
-	int len;
-
-	if (trans) {
-	    /* set send socket buffer if we are transmitting */    
-	    if (setsockopt(fd, SOL_SOCKET, SO_SNDBUF, &sockbufsize,
-			   sizeof sockbufsize) < 0)
-		err("setsockopt: sndbuf");
-	    len = sizeof sockbufsize;
-	    if (getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &sockbufsize, &len) < 0)
-		perror("getsockopt: sndbuf");
-	    mes("sndbuf");
-	} else {
-	    /* set receive socket buffer if we are receiving */    
-	    if (setsockopt(fd, SOL_SOCKET, SO_RCVBUF, &sockbufsize,
-			   sizeof sockbufsize) < 0)
-		err("setsockopt: rcvbuf");
-	    len = sizeof sockbufsize;
-	    if (getsockopt(fd, SOL_SOCKET, SO_RCVBUF, &sockbufsize, &len) < 0)
-		perror("getsockopt: rcvbuf");
-	    mes("rcvbuf");
-	}
-	printf("real buffer size = %d\n",sockbufsize);
-    }
-#endif
 
     if (!atm || satm.sas_family == AF_ATMPVC || !trans)
-	if (bind(fd, atm ? &satm : &sinme, atm ? satm.sas_family == AF_ATMPVC ?
+	if (bind(fd, atm ? (struct sockaddr *) &satm : (struct sockaddr *) &sinme, atm ? satm.sas_family == AF_ATMPVC ?
 	  sizeof(struct sockaddr_atmpvc) : sizeof(struct sockaddr_atmsvc) :
 	  sizeof(sinme)) < 0)
 	    err("bind");
@@ -453,7 +432,7 @@ int no_check = 0;
 			err("setsockopt");
 	    }
 	    
-	    if (connect(fd, atm ? &satm : &sinhim, atm ? sizeof(satm) :
+	    if (connect(fd, atm ? (struct sockaddr *) &satm : (struct sockaddr *) &sinhim, atm ? sizeof(satm) :
 	      sizeof(sinhim)) < 0)
 		err("connect");
 	    
@@ -482,13 +461,13 @@ int no_check = 0;
 	    fromlen = sizeof(frominet);
 	    domain = AF_INET;
 	    
-	    if ((fd=accept(fd, &frominet, &fromlen) ) < 0)
+	    if ((fd=accept(fd, (struct sockaddr *) &frominet, &fromlen) ) < 0)
 		err("accept");
 	    
 	    {
 		struct sockaddr_atmsvc peer;
 		int peerlen = sizeof(peer);
-		if (getpeername(fd, (struct sockaddr_in *) &peer, 
+		if (getpeername(fd, (struct sockaddr *) &peer, 
 				&peerlen) < 0) {
 		    err("getpeername");
 		}
@@ -496,7 +475,7 @@ int no_check = 0;
 		    char name[MAX_ATM_ADDR_LEN+1];
 
 		    if (atm2text(name,MAX_ATM_ADDR_LEN+1,(struct sockaddr *)
-		      &peer,A2T_NAME | A2T_PRETTY) < 0)
+		      &peer,/* A2T_NAME | */ A2T_PRETTY) < 0)
 			strcpy(name,"<invalid>");
 		    fprintf(stderr,"ttcp-r: accept from %s\n",name);
 		}
@@ -515,6 +494,35 @@ int no_check = 0;
 	    }
 	}
     }
+
+    /* set socket buffer size */
+#if defined(SO_SNDBUF) || defined(SO_RCVBUF)
+    if (sockbufsize) {
+	int len;
+
+	if (trans) {
+	    /* set send socket buffer if we are transmitting */    
+	    if (setsockopt(fd, SOL_SOCKET, SO_SNDBUF, &sockbufsize,
+			   sizeof sockbufsize) < 0)
+		err("setsockopt: sndbuf");
+	    len = sizeof sockbufsize;
+	    if (getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &sockbufsize, &len) < 0)
+		perror("getsockopt: sndbuf");
+	    mes("sndbuf");
+        } else {
+	    /* set receive socket buffer if we are receiving */    
+	    if (setsockopt(fd, SOL_SOCKET, SO_RCVBUF, &sockbufsize,
+			   sizeof sockbufsize) < 0)
+		err("setsockopt: rcvbuf");
+	    len = sizeof sockbufsize;
+	    if (getsockopt(fd, SOL_SOCKET, SO_RCVBUF, &sockbufsize, &len) < 0)
+		perror("getsockopt: rcvbuf");
+	    mes("rcvbuf");
+	}
+	printf("real buffer size = %d\n",sockbufsize);
+    }
+#endif
+
     if (tos) {
 	unsigned char tos_value;
 
@@ -613,7 +621,7 @@ int no_check = 0;
 	(void)Nwrite(fd, buf, 4);	/* rcvr end */
 	(void)Nwrite(fd, buf, 4);	/* rcvr end */
     }
-    
+
     if (cput <= 0.0)
 	cput = 0.001;
     
@@ -638,16 +646,20 @@ int no_check = 0;
 		nbytes, cput, outfmt((double)nbytes/cput));
 
 	fprintf(stdout,
-		"ttcp%s: %d I/O calls, msec/call = %.2f, calls/sec = %.2f\n",
+		"ttcp%s: %ld I/O calls, msec/call = %.2f, calls/sec = %.2f\n",
 		trans?"-t":"-r",
 		numCalls,
 		1000.0 * realt/((double)numCalls),
 		((double)numCalls)/realt);
 	fprintf(stdout,
-		"ttcp%s: buffer address %#x\n",
+		"ttcp%s: buffer address %#lx\n",
 		trans?"-t":"-r",
-		buf);
+		(unsigned long )buf);
     }
+
+    if (udp && trans)
+	sleep(1);
+
     exit(0);
 
   usage:
@@ -656,8 +668,7 @@ int no_check = 0;
 }
 
 void
-err(s)
-char *s;
+err(const char *s)
 {
 	int	en = errno;
 
@@ -670,8 +681,7 @@ char *s;
 }
 
 void
-mes(s)
-char *s;
+mes(const char *s)
 {
 	fprintf(stderr,"ttcp%s: %s\n", trans?"-t":"-r", s);
 }
@@ -681,6 +691,7 @@ char *s;
  * pattern - 
  *-------------------------------------------------------------------------
  */
+void
 pattern(cp, cnt)
 register char *cp;
 register int cnt;
@@ -758,7 +769,7 @@ gettimeofday(tp, zp)
  *			P R E P _ T I M E R
  */
 void
-prep_timer()
+prep_timer(void)
 {
 	gettimeofday(&start_time, (struct timezone *)0);
 	getrusage(RUSAGE_SELF, &ru0);
@@ -770,6 +781,7 @@ prep_timer()
  */
 double read_timer(str,len)
 char *str;
+int len;
 {
     struct rusage ru1;
     struct timeval tend, tstart, td;
@@ -813,7 +825,7 @@ prusage(r0, r1, e, b, outp)
 #if defined(sgi)		/* IRIX 3.3 will show 0 for %M,%F,%R,%C */
 	cp = "%Uuser %Ssys %Ereal %P %Mmaxrss %F+%Rpf %Ccsw";
 #else
-	cp = "%Uuser %Ssys %Ereal %P %Xi+%Dd %Mmaxrss %F+%Rpf %Ccsw";
+	cp = (char *) "%Uuser %Ssys %Ereal %P %Xi+%Dd %Mmaxrss %F+%Rpf %Ccsw";
 #endif
 #endif
 	for (; *cp; cp++)  {
@@ -823,13 +835,13 @@ prusage(r0, r1, e, b, outp)
 
 		case 'U':
 			tvsub(&tdiff, &r1->ru_utime, &r0->ru_utime);
-			sprintf(outp,"%d.%01d", tdiff.tv_sec, tdiff.tv_usec/100000);
+			sprintf(outp,"%ld.%01ld", tdiff.tv_sec, tdiff.tv_usec/100000);
 			END(outp);
 			break;
 
 		case 'S':
 			tvsub(&tdiff, &r1->ru_stime, &r0->ru_stime);
-			sprintf(outp,"%d.%01d", tdiff.tv_sec, tdiff.tv_usec/100000);
+			sprintf(outp,"%ld.%01ld", tdiff.tv_sec, tdiff.tv_usec/100000);
 			END(outp);
 			break;
 
@@ -851,49 +863,49 @@ prusage(r0, r1, e, b, outp)
 			break;
 
 		case 'X':
-			sprintf(outp,"%d", t == 0 ? 0 : (r1->ru_ixrss-r0->ru_ixrss)/t);
+			sprintf(outp,"%ld", t == 0 ? 0 : (r1->ru_ixrss-r0->ru_ixrss)/t);
 			END(outp);
 			break;
 
 		case 'D':
-			sprintf(outp,"%d", t == 0 ? 0 :
+			sprintf(outp,"%ld", t == 0 ? 0 :
 			    (r1->ru_idrss+r1->ru_isrss-(r0->ru_idrss+r0->ru_isrss))/t);
 			END(outp);
 			break;
 
 		case 'K':
-			sprintf(outp,"%d", t == 0 ? 0 :
+			sprintf(outp,"%ld", t == 0 ? 0 :
 			    ((r1->ru_ixrss+r1->ru_isrss+r1->ru_idrss) -
 			    (r0->ru_ixrss+r0->ru_idrss+r0->ru_isrss))/t);
 			END(outp);
 			break;
 
 		case 'M':
-			sprintf(outp,"%d", r1->ru_maxrss/2);
+			sprintf(outp,"%ld", r1->ru_maxrss/2);
 			END(outp);
 			break;
 
 		case 'F':
-			sprintf(outp,"%d", r1->ru_majflt-r0->ru_majflt);
+			sprintf(outp,"%ld", r1->ru_majflt-r0->ru_majflt);
 			END(outp);
 			break;
 
 		case 'R':
-			sprintf(outp,"%d", r1->ru_minflt-r0->ru_minflt);
+			sprintf(outp,"%ld", r1->ru_minflt-r0->ru_minflt);
 			END(outp);
 			break;
 
 		case 'I':
-			sprintf(outp,"%d", r1->ru_inblock-r0->ru_inblock);
+			sprintf(outp,"%ld", r1->ru_inblock-r0->ru_inblock);
 			END(outp);
 			break;
 
 		case 'O':
-			sprintf(outp,"%d", r1->ru_oublock-r0->ru_oublock);
+			sprintf(outp,"%ld", r1->ru_oublock-r0->ru_oublock);
 			END(outp);
 			break;
 		case 'C':
-			sprintf(outp,"%d+%d", r1->ru_nvcsw-r0->ru_nvcsw,
+			sprintf(outp,"%ld+%ld", r1->ru_nvcsw-r0->ru_nvcsw,
 				r1->ru_nivcsw-r0->ru_nivcsw );
 			END(outp);
 			break;
@@ -958,31 +970,34 @@ register char *cp;
 /*
  *			N R E A D
  */
-Nread( fd, buf, count )
-int fd;
-char *buf;
+int
+Nread(_fd, _buf, count)
+int _fd;
+char *_buf;
 int count;
 {
+#if 0
 	struct sockaddr_in from;
 	int len = sizeof(from);
+#endif
 	register int cnt;
 	if( udp )  {
 #if 0
-		cnt = recvfrom( fd, buf, count, 0, &from, &len );
+		cnt = recvfrom(_fd, _buf, count, 0, &from, &len );
 #else
-		cnt = recv( fd, buf, count, 0);
+		cnt = recv(_fd, _buf, count, 0);
 #endif
 		numCalls++;
 	} else {
 		if( b_flag )
-			cnt = mread( fd, buf, count );	/* fill buf */
+			cnt = mread( _fd, _buf, count );	/* fill buf */
 		else {
-			cnt = read( fd, buf, count );
+			cnt = read( _fd, _buf, count );
 			numCalls++;
 		}
 		if (touchdata && cnt > 0) {
 			register int c = cnt, sum;
-			register char *b = buf;
+			register char *b = _buf;
 			while (c--)
 				sum += *b++;
 		}
@@ -994,17 +1009,18 @@ int count;
  * Nwrite - 
  *-------------------------------------------------------------------------
  */
-Nwrite(fd, buf, count)
-int fd;
-char *buf;
+int
+Nwrite(_fd, _buf, count)
+int _fd;
+char *_buf;
 int count;
 {
     register int cnt;
     
     if (udp)  {
       again:
-	if (atm) cnt = write(fd, buf, count);
-	else cnt = sendto(fd, buf, count, 0, &sinhim, sizeof(sinhim));
+	if (atm) cnt = write(_fd, _buf, count);
+	else cnt = sendto(_fd, _buf, count, 0, (struct sockaddr *) &sinhim, sizeof(sinhim));
 	numCalls++;
 	if ( cnt<0 && errno == ENOBUFS )  {
 	    delay(18000);
@@ -1012,7 +1028,7 @@ int count;
 	    goto again;
 	}
     } else {
-	cnt = write(fd, buf, count);
+	cnt = write(_fd, _buf, count);
 	numCalls++;
     }
 if (cnt < 0) perror("WA:write");
@@ -1020,13 +1036,13 @@ if (cnt < 0) perror("WA:write");
 }
 
 void
-delay(us)
+delay(long us)
 {
 	struct timeval tv;
 
 	tv.tv_sec = 0;
 	tv.tv_usec = us;
-	(void)select( 1, (char *)0, (char *)0, (char *)0, &tv );
+	(void)select( 1, (void *)0, (void *)0, (void *)0, &tv );
 }
 
 /*
@@ -1039,8 +1055,8 @@ delay(us)
  * grouping as it is written with.  Written by Robert S. Miles, BRL.
  */
 int
-mread(fd, bufp, n)
-int fd;
+mread(_fd, bufp, n)
+int _fd;
 register char	*bufp;
 unsigned	n;
 {
@@ -1048,7 +1064,7 @@ unsigned	n;
 	register int		nread;
 
 	do {
-		nread = read(fd, bufp, n-count);
+		nread = read(_fd, bufp, n-count);
 		numCalls++;
 		if(nread < 0)  {
 			perror("ttcp_mread");
